==> surface_sim-0.2.0.tar.gz: surface_sim-0.2.0/surface_sim/setup/setup.py <==
from typing import Any, Dict, Type, TypeVar, Union, List
from copy import deepcopy
from pathlib import Path

import yaml

T = TypeVar("T", bound="Setup")


class Setup:
    def __init__(self, setup: Dict[str, Any]) -> None:
        self._qubit_params = dict()
        self._global_params = dict()
        self._var_params = dict()

        _setup = deepcopy(setup)
        self.name = _setup.pop("name", None)
        self.description = _setup.pop("description", None)
        self._gate_durations = _setup.pop("gate_durations", {})
        self._load_setup(_setup)

    def _load_setup(self, setup: Dict[str, Any]) -> None:
        params = setup.get("setup")
        if not params:
            raise ValueError("setup not found or contains no information")

        for params_dict in params:
            if "qubit" in params_dict:
                qubit = str(params_dict.pop("qubit"))
                qubits = (qubit,)
            elif "qubits" in params_dict:
                qubits = tuple(params_dict.pop("qubits"))
            else:
                qubits = None

            if qubits:
                if qubits in self._qubit_params.keys():
                    raise ValueError("Parameters defined repeatedly in the setup.")
                self._qubit_params[qubits] = params_dict
            else:
                self._global_params.update(params_dict)

            for param, val in params_dict.items():
                if isinstance(val, str) and param not in self._var_params:
                    self._var_params[val] = None

    @property
    def free_params(self) -> List[str]:
        return [param for param, val in self._var_params.items() if val is None]

    @classmethod
    def from_yaml(cls: Type[T], filename: Union[str, Path]) -> T:
        """
        from_yaml Create new surface_sim.setup.Setup instance from YAML configuarion file.

        Parameters
        ----------
        filename : str
            The YAML file name.

        Returns
        -------
        T
            The initialised surface_sim.setup.Setup object based on the yaml.
        """
        with open(filename, "r") as file:
            setup = yaml.safe_load(file)
            return cls(setup)

    def to_dict(self) -> Dict[str, Any]:
        setup = dict()

        setup["name"] = self.name
        setup["description"] = self.description
        setup["gate_durations"] = self._gate_durations

        qubit_params = []
        if self._global_params:
            qubit_params.append(self._global_params)

        for qubits, params in self._qubit_params.items():
            params_copy = deepcopy(params)
            num_qubits = len(qubits)
            if num_qubits == 1:
                params_copy["qubit"] = qubits[0]
            elif num_qubits == 2:
                params_copy["qubits"] = list(qubits)
            qubit_params.append(params_copy)

        setup["setup"] = qubit_params

        return setup

    def to_yaml(self, filename: Union[str, Path]) -> None:
        setup = self.to_dict()

        with open(filename, "w") as file:
            yaml.dump(setup, file, default_flow_style=False)

    def var_param(self, var_param: str) -> float:
        try:
            return self._var_params[var_param]
        except KeyError:
            raise ValueError(f"Variable param {var_param} not in setup.free_params.")

    def set_var_param(self, var_param: str, val: float) -> None:
        try:
            self._var_params[var_param] = val
        except KeyError:
            raise ValueError(f"Variable param {var_param} not in setup.")

    def set_param(self, param: str, param_val: float, *qubits: str) -> None:
        if not qubits:
            self._global_params[param] = param_val
        else:
            self._qubit_params[qubits][param] = param_val

    def param(self, param: str, *qubits: str) -> float:
        if qubits in self._qubit_params and param in self._qubit_params[qubits]:
            val = self._qubit_params[qubits][param]
            return self._eval_param_val(val)
        if param in self._global_params:
            val = self._global_params[param]
            return self._eval_param_val(val)

        if qubits:
            qubit_str = ", ".join(qubits)
            raise KeyError(f"Parameter {param} not defined for qubit(s) {qubit_str}")
        raise KeyError(f"Global parameter {param} not defined")

    def _eval_param_val(self, val):
        if val in self._var_params:
            param = deepcopy(val)
            val = self._var_params[param]
            if val is None:
                raise ValueError(f"Variable parameter {param} is not specified.")

        return val

    def gate_duration(self, name: str) -> float:
        try:
            return self._gate_durations[name]
        except KeyError:
            raise ValueError(f"No gate duration specified for '{name}'")

==> surface_sim-0.3.0.tar.gz: surface_sim-0.3.0/surface_sim/setup/setup.py <==
from __future__ import annotations
from copy import deepcopy
from pathlib import Path

import yaml


class Setup:
    def __init__(self, setup: dict[str, object]) -> None:
        """Initialises teh ``Setup`` class.

        Parameters
        ----------
        setup
            Dictionary with the configuration.
            Must have the key ``"setup"`` containing the information.
            It can also include ``"name"``, ``"description"`` and
            ``"gate_durations"`` keys with the corresponding information.
        """
        self._qubit_params = dict()
        self._global_params = dict()
        self._var_params = dict()

        _setup = deepcopy(setup)
        self.name = _setup.pop("name", None)
        self.description = _setup.pop("description", None)
        self._gate_durations = _setup.pop("gate_durations", {})
        self._load_setup(_setup)
        return

    def _load_setup(self, setup: dict[str, object]) -> None:
        params = setup.get("setup")
        if not params:
            raise ValueError("'setup['setup']' not found or contains no information.")

        for params_dict in params:
            if "qubit" in params_dict:
                qubit = str(params_dict.pop("qubit"))
                qubits = (qubit,)
            elif "qubits" in params_dict:
                qubits = tuple(params_dict.pop("qubits"))
            else:
                qubits = None

            if qubits:
                if qubits in self._qubit_params.keys():
                    raise ValueError("Parameters defined repeatedly in the setup.")
                self._qubit_params[qubits] = params_dict
            else:
                self._global_params.update(params_dict)

            for param, val in params_dict.items():
                if isinstance(val, str) and param not in self._var_params:
                    self._var_params[val] = None

    @property
    def free_params(self) -> list[str]:
        """Returns the unset variable parameters."""
        return [param for param, val in self._var_params.items() if val is None]

    @classmethod
    def from_yaml(cls: type[Setup], filename: str | Path) -> Setup:
        """Create new ``surface_sim.setup.Setup`` instance from YAML
        configuarion file.

        Parameters
        ----------
        filename
            The YAML file name.

        Returns
        -------
        T
            The initialised ``surface_sim.setup.Setup`` object based on the yaml.
        """
        with open(filename, "r") as file:
            setup = yaml.safe_load(file)
            return cls(setup)

    def to_dict(self) -> dict[str, object]:
        """Returns a dictionary that can be used to initialize ``Setup``."""
        setup = dict()

        setup["name"] = self.name
        setup["description"] = self.description
        setup["gate_durations"] = self._gate_durations

        qubit_params = []
        if self._global_params:
            qubit_params.append(self._global_params)

        for qubits, params in self._qubit_params.items():
            params_copy = deepcopy(params)
            num_qubits = len(qubits)
            if num_qubits == 1:
                params_copy["qubit"] = qubits[0]
            elif num_qubits == 2:
                params_copy["qubits"] = list(qubits)
            qubit_params.append(params_copy)

        setup["setup"] = qubit_params

        return setup

    def to_yaml(self, filename: str | Path) -> None:
        """Stores the current ``Setup`` configuration in the given file
        in YAML format.

        Parameters
        ----------
        filename
            Name of the file in which to store the configuration.
        """
        setup = self.to_dict()

        with open(filename, "w") as file:
            yaml.dump(setup, file, default_flow_style=False)
        return

    def var_param(self, var_param: str) -> float:
        """Returns the value of the given variable parameter name.

        Parameters
        ----------
        var_param
            Name of the variable parameter.

        Returns
        -------
        Value of the specified ``var_param``.
        """
        try:
            return self._var_params[var_param]
        except KeyError:
            raise ValueError(f"Variable param {var_param} not in setup.free_params.")

    def set_var_param(self, var_param: str, val: float) -> None:
        """Sets the given value to the given variable parameter.

        Parameters
        ----------
        var_param
            Name of the variable parameter.
        val
            Value to set to ``var_param``.
        """
        try:
            self._var_params[var_param] = val
        except KeyError:
            raise ValueError(f"Variable param {var_param} not in setup.")

    def set_param(self, param: str, param_val: float, *qubits: str) -> None:
        """Sets the given value to the given parameter of the given qubit(s).

        Parameters
        ----------
        param
            Name of the parameter.
        param_val
            Value to set to ``param``.
        *qubits
            Qubit or qubit pairs of which to set the parameter.
        """
        if not qubits:
            self._global_params[param] = param_val
        else:
            self._qubit_params[qubits][param] = param_val

    def param(self, param: str, *qubits: str) -> float:
        """Returns the value of the given parameter for the specified qubit(s).

        Parameters
        ----------
        param
            Name of the parameter.
        *qubits
            Qubit or qubit pairs of which to set the parameter.

        Returns
        -------
        val
            Value of the parameter.
        """
        if qubits in self._qubit_params and param in self._qubit_params[qubits]:
            val = self._qubit_params[qubits][param]
            return self._eval_param_val(val)
        if param in self._global_params:
            val = self._global_params[param]
            return self._eval_param_val(val)

        if qubits:
            qubit_str = ", ".join(qubits)
            raise KeyError(f"Parameter {param} not defined for qubit(s) {qubit_str}")
        raise KeyError(f"Global parameter {param} not defined")

    def _eval_param_val(self, val):
        if val in self._var_params:
            param = deepcopy(val)
            val = self._var_params[param]
            if val is None:
                raise ValueError(f"Variable parameter {param} is not specified.")

        return val

    def gate_duration(self, name: str) -> float:
        """Returns the duration of the specified gate.

        Parameters
        ----------
        name
            Name of the gate.

        Returns
        -------
        Duration of the gate.
        """
        try:
            return self._gate_durations[name]
        except KeyError:
            raise ValueError(f"No gate duration specified for '{name}'")
